import { CreateLogger, BaseLogger } from '@stenodb/logger';
import { ClassEntity, EntityTransformer, DataTransformer } from '@stenodb/utils';

declare class SessionStorage<T> extends BrowserStorage<T> {
    constructor(name: string, entity: Steno.Entity<T>, initialData?: T);
}

declare class StorageProvider<T> {
    #private;
    constructor(adapter: Steno.BrowserAdapter<T>);
    get data(): T | null;
    set data(data: T | null);
    get initialData(): T | null;
    set initialData(data: T | undefined | null);
    read(): T | null;
    write(): void;
    reset(): void;
    exists(): boolean;
}

declare namespace Steno {
    type Entity<T> = ClassEntity<T>;
    type BrowserAdapter<T> = LocalStorage<T> | SessionStorage<T>;
    type BrowserProvider<T> = StorageProvider<T>;
    interface BrowserProviderOptions {
        logger?: CreateLogger;
    }
}

declare class BrowserStorage<T> {
    name: string;
    storage: Storage;
    logger: BaseLogger | undefined;
    entityTransformer: EntityTransformer<T>;
    dataTransformer: DataTransformer<T>;
    data: T | null;
    initialData: T | null;
    constructor(name: string, storage: Storage, entity: Steno.Entity<T>, initialData?: T);
    registerLogger(logger: BaseLogger): void;
    read(): void;
    write(): void;
    reset(): void;
    exists(): boolean;
}

declare class LocalStorage<T> extends BrowserStorage<T> {
    constructor(name: string, entity: Steno.Entity<T>, initialData?: T);
}

declare class BrowserProvider {
    #private;
    constructor(options?: Steno.BrowserProviderOptions);
    private registerAdapterModules;
    create<T>(adapter: Steno.BrowserAdapter<T>): StorageProvider<T>;
}

export { BrowserProvider, LocalStorage, SessionStorage, Steno };
