var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var __accessCheck = (obj, member, msg) => {
  if (!member.has(obj))
    throw TypeError("Cannot " + msg);
};
var __privateGet = (obj, member, getter) => {
  __accessCheck(obj, member, "read from private field");
  return getter ? getter.call(obj) : member.get(obj);
};
var __privateAdd = (obj, member, value) => {
  if (member.has(obj))
    throw TypeError("Cannot add the same private member more than once");
  member instanceof WeakSet ? member.add(obj) : member.set(obj, value);
};
var __privateSet = (obj, member, value, setter) => {
  __accessCheck(obj, member, "write to private field");
  setter ? setter.call(obj, value) : member.set(obj, value);
  return value;
};

// src/adapter/WebStorage.ts
import { dataTransformer, entityTransformer } from "@stenodb/utils";
var BrowserStorage = class {
  constructor(name, storage, entity, initialData) {
    __publicField(this, "name");
    __publicField(this, "storage");
    __publicField(this, "logger");
    __publicField(this, "entityTransformer");
    __publicField(this, "dataTransformer");
    __publicField(this, "data", null);
    __publicField(this, "initialData", null);
    this.name = name;
    this.storage = storage;
    this.entityTransformer = entityTransformer(entity);
    this.dataTransformer = dataTransformer(this.entityTransformer);
    if (initialData) {
      this.initialData = initialData;
    }
  }
  registerLogger(logger) {
    this.logger = logger;
  }
  read() {
    const data = this.storage.getItem(this.name);
    this.data = this.dataTransformer.toJSON(data);
    this.logger?.info("Read data:", this.data);
  }
  write() {
    this.storage.setItem(this.name, this.dataTransformer.toString(this.data));
    this.logger?.info("Write data:", this.data);
  }
  reset() {
    if (!this.initialData)
      return;
    this.data = this.entityTransformer(this.initialData);
    this.write();
  }
  exists() {
    return this.read() !== null;
  }
};

// src/adapter/LocalStorage.ts
var LocalStorage = class extends BrowserStorage {
  constructor(name, entity, initialData) {
    super(name, localStorage, entity, initialData);
  }
};

// src/adapter/SessionStorage.ts
var SessionStorage = class extends BrowserStorage {
  constructor(name, entity, initialData) {
    super(name, sessionStorage, entity, initialData);
  }
};

// src/provider/StorageProvider.ts
var _adapter;
var StorageProvider = class {
  constructor(adapter) {
    __privateAdd(this, _adapter, void 0);
    __privateSet(this, _adapter, adapter);
  }
  get data() {
    return __privateGet(this, _adapter).data;
  }
  set data(data) {
    __privateGet(this, _adapter).data = data;
  }
  get initialData() {
    return __privateGet(this, _adapter).initialData;
  }
  set initialData(data) {
    if (!data)
      return;
    __privateGet(this, _adapter).initialData = data;
  }
  read() {
    __privateGet(this, _adapter).read();
    if (!__privateGet(this, _adapter).data) {
      this.reset();
    }
    return __privateGet(this, _adapter).data;
  }
  write() {
    __privateGet(this, _adapter).write();
  }
  reset() {
    __privateGet(this, _adapter).reset();
  }
  exists() {
    return __privateGet(this, _adapter).exists();
  }
};
_adapter = new WeakMap();

// src/provider/BrowserProvider.ts
var _options;
var BrowserProvider = class {
  constructor(options = {}) {
    __privateAdd(this, _options, void 0);
    __privateSet(this, _options, options);
  }
  registerAdapterModules(adapter) {
    if (!__privateGet(this, _options)?.logger)
      return;
    const logger = __privateGet(this, _options).logger(adapter.name);
    adapter.registerLogger(logger);
  }
  create(adapter) {
    this.registerAdapterModules(adapter);
    return new StorageProvider(adapter);
  }
};
_options = new WeakMap();
export {
  BrowserProvider,
  LocalStorage,
  SessionStorage
};
